@@ -122,6 +122,47 @@ class KeyPairRSA extends KeyPair {
     return plain;
   }
 
+  @Override
+  byte[] getOpenSSHv1PrivateKeyBlob() {
+    byte[] keyTypeName = getKeyTypeName();
+    if (keyTypeName == null || n_array == null || pub_array == null || prv_array == null
+        || c_array == null || p_array == null || q_array == null) {
+      return null;
+    }
+
+    Buffer _buf = null;
+    try {
+      int _bufLen = 4 + keyTypeName.length;
+      _bufLen += 4 + n_array.length;
+      _bufLen += (n_array[0] & 0x80) >>> 7;
+      _bufLen += 4 + pub_array.length;
+      _bufLen += (pub_array[0] & 0x80) >>> 7;
+      _bufLen += 4 + prv_array.length;
+      _bufLen += (prv_array[0] & 0x80) >>> 7;
+      _bufLen += 4 + c_array.length;
+      _bufLen += (c_array[0] & 0x80) >>> 7;
+      _bufLen += 4 + p_array.length;
+      _bufLen += (p_array[0] & 0x80) >>> 7;
+      _bufLen += 4 + q_array.length;
+      _bufLen += (q_array[0] & 0x80) >>> 7;
+      _buf = new Buffer(_bufLen);
+      _buf.putString(keyTypeName);
+      _buf.putMPInt(n_array);
+      _buf.putMPInt(pub_array);
+      _buf.putMPInt(prv_array);
+      _buf.putMPInt(c_array);
+      _buf.putMPInt(p_array);
+      _buf.putMPInt(q_array);
+
+      return _buf.buffer;
+    } catch (Exception e) {
+      if (_buf != null) {
+        Util.bzero(_buf.buffer);
+      }
+      throw e;
+    }
+  }
+
   @Override
   boolean parse(byte[] plain) throws JSchException {
 
@@ -179,23 +220,22 @@ class KeyPairRSA extends KeyPair {
 
       // OPENSSH Key v1 Format
       if (vendor == VENDOR_OPENSSH_V1) {
-        final Buffer prvKEyBuffer = new Buffer(plain);
-        int checkInt1 = prvKEyBuffer.getInt(); // uint32 checkint1
-        int checkInt2 = prvKEyBuffer.getInt(); // uint32 checkint2
+        Buffer prvKeyBuffer = new Buffer(plain);
+        int checkInt1 = prvKeyBuffer.getInt(); // uint32 checkint1
+        int checkInt2 = prvKeyBuffer.getInt(); // uint32 checkint2
         if (checkInt1 != checkInt2) {
           throw new JSchException("check failed");
         }
-        String keyType = Util.byte2str(prvKEyBuffer.getString()); // string keytype
-        n_array = prvKEyBuffer.getMPInt(); // Modulus
-        pub_array = prvKEyBuffer.getMPInt(); // Public Exponent
-        prv_array = prvKEyBuffer.getMPInt(); // Private Exponent
-        c_array = prvKEyBuffer.getMPInt(); // iqmp (q^-1 mod p)
-        p_array = prvKEyBuffer.getMPInt(); // p (Prime 1)
-        q_array = prvKEyBuffer.getMPInt(); // q (Prime 2)
-        if (n_array != null) {
-          key_size = (new BigInteger(n_array)).bitLength();
-        }
-        publicKeyComment = Util.byte2str(prvKEyBuffer.getString());
+
+        String keyType = Util.byte2str(prvKeyBuffer.getString()); // string keytype
+        n_array = prvKeyBuffer.getMPInt(); // Modulus
+        pub_array = prvKeyBuffer.getMPInt(); // Public Exponent
+        prv_array = prvKeyBuffer.getMPInt(); // Private Exponent
+        c_array = prvKeyBuffer.getMPInt(); // iqmp (q^-1 mod p)
+        p_array = prvKeyBuffer.getMPInt(); // p (Prime 1)
+        q_array = prvKeyBuffer.getMPInt(); // q (Prime 2)
+        key_size = (new BigInteger(n_array)).bitLength();
+        publicKeyComment = Util.byte2str(prvKeyBuffer.getString());
 
         getEPArray();
         getEQArray();
