@@ -171,7 +171,7 @@ public class SftpATTRS {
   int mtime;
   String[] extended = null;
 
-  private SftpATTRS() {}
+  public SftpATTRS() {}
 
   static SftpATTRS getATTR(Buffer buf) {
     SftpATTRS attr = new SftpATTRS();
