@@ -26,6 +26,7 @@
 
 package com.turbovnc.rfb;
 
+import java.awt.event.*;
 import java.io.*;
 import java.util.*;
 
@@ -138,7 +139,51 @@ public final class Params {
     return null;
   }
 
-  public void list(int width, boolean advanced) {
+  private static String wrapText(String str, int width, int offset) {
+    String[] words = str.split(" ");
+    int len = 0;
+    StringBuilder sb = new StringBuilder();
+
+    for (int i = 0; i < offset; i++)
+      sb.append(" ");
+    for (String word : words) {
+      if (len + word.length() + 1 <= width - offset) {
+        sb.append(word);
+        len += word.length() + 1;
+      } else {
+        if (sb.charAt(sb.length() - 1) == ' ')
+          sb.deleteCharAt(sb.length() - 1);
+        sb.append("\n");
+        for (int i = 0; i < offset; i++)
+          sb.append(" ");
+        if (word.length() <= 0) {
+          len = 0;
+          continue;
+        }
+        sb.append(word);
+        len = word.length() + 1;
+      }
+      if (word.length() > 0 && word.charAt(word.length() - 1) == '\n') {
+        sb.append("\n");
+        for (int i = 0; i < offset; i++)
+          sb.append(" ");
+        len = 0;
+      } else if (word.length() > 0 && word.charAt(word.length() - 1) == '\r') {
+        sb.deleteCharAt(sb.length() - 1);
+        sb.append("\n");
+        for (int i = 0; i < offset; i++)
+          sb.append(" ");
+        len = 0;
+      } else
+        sb.append(" ");
+    }
+    if (sb.charAt(sb.length() - 1) == ' ')
+      sb.deleteCharAt(sb.length() - 1);
+
+    return sb.toString();
+  }
+
+  public void list(boolean advanced) {
     VoidParameter current = head;
 
     while (current != null) {
@@ -162,44 +207,18 @@ public final class Params {
         continue;
       }
 
-      System.out.print("--> " + current.getName() + "\n    ");
-      if (current.getValues() != null)
-        System.out.print("Values: " + current.getValues() + " ");
-      if (current.getDefaultStr() != null)
-        System.out.print("(default = " + current.getDefaultStr() + ")\n");
-      System.out.print("\n   ");
+      System.out.print("--> " + current.getName() + "\n");
+      String valuesStr = (current.getValues() != null ?
+                          "Values: " + current.getValues() + " " : "") +
+                         (current.getDefaultStr() != null ?
+                          "(default = " + current.getDefaultStr() + ")" : "");
+      if (valuesStr.length() > 0)
+        System.out.println(wrapText(valuesStr, 80, 4));
+      System.out.print("\n");
+      System.out.println(wrapText(desc, 80, 4));
+      System.out.print("\n");
 
-      int column = 4;
-      while (true) {
-        int s = desc.indexOf(' ');
-        while (desc.charAt(s + 1) == ' ') s++;
-        int wordLen;
-        if (s > -1) wordLen = s;
-        else wordLen = desc.length();
-
-        if (column + wordLen + 1 > width) {
-          System.out.print("\n   ");
-          column = 4;
-        }
-        if (wordLen >= 1 && desc.charAt(wordLen - 1) == '\r') {
-          System.out.format("%" + wordLen + "s",
-                            desc.substring(0, wordLen - 1));
-          column += wordLen;
-        } else {
-          System.out.format(" %" + wordLen + "s", desc.substring(0, wordLen));
-          column += wordLen + 1;
-        }
-        if (wordLen >= 1 && (desc.charAt(wordLen - 1) == '\n' ||
-                             desc.charAt(wordLen - 1) == '\r')) {
-          System.out.print("\n   ");
-          column = 4;
-        }
-
-        if (s == -1) break;
-        desc = desc.substring(wordLen + 1);
-      }
       current = current.next();
-      System.out.print("\n\n");
     }
   }
 
@@ -684,10 +703,11 @@ public final class Params {
   "\"Always\" = Automatically grab the keyboard in both full-screen mode " +
   "and windowed mode.\r " +
   "\"Manual\" = Only grab/ungrab the keyboard when the \"Grab Keyboard\" F8 " +
-  "menu option is selected or the Ctrl-Alt-Shift-G hotkey is pressed.\n " +
+  "menu option is selected or the corresponding hotkey is pressed.\n " +
 
-  "Regardless of the grabbing mode, the F8 menu option and hotkey can " +
-  "always be used to grab or ungrab the keyboard.", GrabParameter.FS);
+  "Regardless of the grabbing mode, the F8 menu option and corresponding " +
+  "hotkey can always be used to grab or ungrab the keyboard.",
+  GrabParameter.FS);
 
   public BoolParameter grabPointer =
   new BoolParameter("GrabPointer", this, false, false,
@@ -699,22 +719,29 @@ public final class Params {
   "cannot switch to other running applications.)  Thus, this parameter is " +
   "primarily useful with GrabKeyboard=FS." : null, true);
 
+  public ModifierParameter hotkeyModifiers =
+  new ModifierParameter("HotkeyModifiers", this,
+  "The combination of modifier keys used to trigger the various hotkey " +
+  "sequences.  Shift cannot be used as a modifier unless it is accompanied " +
+  "by another modifier.",
+  InputEvent.CTRL_DOWN_MASK | InputEvent.ALT_DOWN_MASK |
+  InputEvent.SHIFT_DOWN_MASK);
+
   public MenuKeyParameter menuKey =
   new MenuKeyParameter("MenuKey", this,
   "The key used to display the popup menu", "F8");
 
   public BoolParameter noHotkeys =
   new BoolParameter("NoHotkeys", this, false, true,
-  "Setting this parameter disables all Ctrl-Alt-Shift hotkeys, thus " +
-  "allowing those key sequences to be transmitted to the VNC server.",
-  false);
+  "Setting this parameter disables all hotkeys, thus allowing those key " +
+  "sequences to be transmitted to the VNC server.", false);
 
   public BoolParameter noMacHotkeys =
   new BoolParameter("NoMacHotkeys", this, false, true,
   Utils.isMac() ? "On macOS, the TurboVNC Viewer normally assigns " +
-  "equivalent Command hotkeys for most of its Ctrl-Alt-Shift hotkeys.  " +
-  "However, since the Command key maps to the Super/Meta key on Un*x " +
-  "systems, those Command hotkeys may interfere with hotkeys used by " +
+  "equivalent Command key menu shortcuts (AKA accelerators) for most of its " +
+  "hotkeys.  However, since the Command key maps to the Super/Meta key on " +
+  "Un*x systems, those menu shortcuts may interfere with hotkeys used by " +
   "certain applications (such as Emacs) on the remote system.  Setting this " +
   "parameter allows as many Command key sequences as possible to be " +
   "transmitted to the VNC server as Super/Meta key sequences, although some " +
