@@ -291,6 +291,10 @@ Bool rfbCloseScreen(ScreenPtr pScreen)
 #endif
 
   pScreen->CloseScreen = prfb->CloseScreen;
+#ifdef DRI3
+  if (driNode)
+    pScreen->SourceValidate = prfb->SourceValidate;
+#endif
   pScreen->CreateGC = prfb->CreateGC;
   pScreen->CopyWindow = prfb->CopyWindow;
   pScreen->ClearToBackground = prfb->ClearToBackground;
@@ -299,6 +303,15 @@ Bool rfbCloseScreen(ScreenPtr pScreen)
   if (ps) {
     ps->Composite = prfb->Composite;
     ps->Glyphs = prfb->Glyphs;
+#ifdef DRI3
+    if (driNode) {
+      ps->CompositeRects = prfb->CompositeRects;
+      ps->Trapezoids = prfb->Trapezoids;
+      ps->Triangles = prfb->Triangles;
+      ps->TriStrip = prfb->TriStrip;
+      ps->TriFan = prfb->TriFan;
+    }
+#endif
   }
 #endif
   pScreen->InstallColormap = prfb->InstallColormap;
@@ -313,6 +326,26 @@ Bool rfbCloseScreen(ScreenPtr pScreen)
 }
 
 
+#ifdef DRI3
+
+void rfbSourceValidate(DrawablePtr pDrawable, int x, int y, int width,
+                       int height, unsigned int subWindowMode)
+{
+  SCREEN_PROLOGUE(pDrawable->pScreen, SourceValidate);
+
+  TRC("rfbSourceValidate() Drawable=0x%.8x x=%d y=%d width=%d height=%d subWindowMode=%u\n",
+      XID(pDrawable), x, y, width, height, subWindowMode);
+
+  rfbDRI3SyncBOToDrawable(pDrawable);
+
+  (*pScreen->SourceValidate) (pDrawable, x, y, width, height, subWindowMode);
+
+  SCREEN_EPILOGUE(SourceValidate, rfbSourceValidate);
+}
+
+#endif
+
+
 /*
  * CreateGC - wrap the GC funcs (the GC ops will be wrapped when the GC
  * func "ValidateGC" is called).
@@ -606,8 +639,16 @@ static void rfbFillSpans(DrawablePtr pDrawable, GCPtr pGC,
     TRC_REGION(pDrawable->pScreen, pGC->pCompositeClip, "rfbFillSpans() mod");
   }
 
+#ifdef DRI3
+  rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
+
   (*pGC->ops->FillSpans) (pDrawable, pGC, nInit, pptInit, pwidthInit, fSorted);
 
+#ifdef DRI3
+  rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
+
   SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
 
   GC_OP_EPILOGUE(pGC);
@@ -641,8 +682,16 @@ static void rfbSetSpans(DrawablePtr pDrawable, GCPtr pGC, char *psrc,
     TRC_REGION(pDrawable->pScreen, pGC->pCompositeClip, "rfbSetSpans() mod");
   }
 
+#ifdef DRI3
+  rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
+
   (*pGC->ops->SetSpans) (pDrawable, pGC, psrc, ppt, pwidth, nspans, fSorted);
 
+#ifdef DRI3
+  rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
+
   SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
 
   GC_OP_EPILOGUE(pGC);
@@ -683,9 +732,17 @@ static void rfbPutImage(DrawablePtr pDrawable, GCPtr pGC, int depth,
 
   REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
 
+#ifdef DRI3
+  rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
+
   (*pGC->ops->PutImage) (pDrawable, pGC, depth, x, y, w, h, leftPad, format,
                          pBits);
 
+#ifdef DRI3
+  rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
+
   SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
 
   GC_OP_EPILOGUE(pGC);
@@ -760,8 +817,16 @@ static RegionPtr rfbCopyArea(DrawablePtr pSrc, DrawablePtr pDst, GCPtr pGC,
 
   REGION_UNINIT(pDst->pScreen, &dstRegion);
 
+#ifdef DRI3
+  rfbDRI3SyncBOToDrawable(pDst);
+#endif
+
   rgn = (*pGC->ops->CopyArea) (pSrc, pDst, pGC, srcx, srcy, w, h, dstx, dsty);
 
+#ifdef DRI3
+  rfbDRI3SyncDrawableToBO(pDst);
+#endif
+
   SCHEDULE_FB_UPDATE(pDst->pScreen, prfb);
 
   GC_OP_EPILOGUE(pGC);
@@ -805,9 +870,17 @@ static RegionPtr rfbCopyPlane(DrawablePtr pSrc, DrawablePtr pDst,
 
   REGION_UNINIT(pDst->pScreen, &tmpRegion);
 
+#ifdef DRI3
+  rfbDRI3SyncBOToDrawable(pDst);
+#endif
+
   rgn = (*pGC->ops->CopyPlane) (pSrc, pDst, pGC, srcx, srcy, w, h,
                                 dstx, dsty, plane);
 
+#ifdef DRI3
+  rfbDRI3SyncDrawableToBO(pDst);
+#endif
+
   SCHEDULE_FB_UPDATE(pDst->pScreen, prfb);
 
   GC_OP_EPILOGUE(pGC);
@@ -873,12 +946,19 @@ static void rfbPolyPoint(DrawablePtr pDrawable, GCPtr pGC,
     TRC_REGION(pDrawable->pScreen, &tmpRegion, "rfbPolyPoint() mod");
 
     REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->PolyPoint) (pDrawable, pGC, mode, npt, pts);
 
-  if (npt)
+  if (npt) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -972,12 +1052,19 @@ static void rfbPolylines(DrawablePtr pDrawable, GCPtr pGC, int mode, int npt,
 
     REGION_DESTROY(pDrawable->pScreen, tmpRegion);
     free((char *)rects);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->Polylines) (pDrawable, pGC, mode, npt, ppts);
 
-  if (npt)
+  if (npt) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1037,12 +1124,19 @@ static void rfbPolySegment(DrawablePtr pDrawable, GCPtr pGC, int nseg,
 
     REGION_DESTROY(pDrawable->pScreen, tmpRegion);
     free((char *)rects);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->PolySegment) (pDrawable, pGC, nseg, segs);
 
-  if (nseg)
+  if (nseg) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1109,12 +1203,19 @@ static void rfbPolyRectangle(DrawablePtr pDrawable, GCPtr pGC, int nrects,
 
     REGION_DESTROY(pDrawable->pScreen, tmpRegion);
     free((char *)regRects);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->PolyRectangle) (pDrawable, pGC, nrects, rects);
 
-  if (nrects)
+  if (nrects) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1163,12 +1264,19 @@ static void rfbPolyArc(DrawablePtr pDrawable, register GCPtr pGC, int narcs,
 
     REGION_DESTROY(pDrawable->pScreen, tmpRegion);
     free((char *)rects);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->PolyArc) (pDrawable, pGC, narcs, arcs);
 
-  if (narcs)
+  if (narcs) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1229,12 +1337,19 @@ static void rfbFillPolygon(register DrawablePtr pDrawable, register GCPtr pGC,
     TRC_REGION(pDrawable->pScreen, &tmpRegion, "rfbFillPolygon() mod");
 
     REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->FillPolygon) (pDrawable, pGC, shape, mode, count, pts);
 
-  if (count)
+  if (count) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1276,12 +1391,19 @@ static void rfbPolyFillRect(DrawablePtr pDrawable, GCPtr pGC, int nrects,
 
     REGION_DESTROY(pDrawable->pScreen, tmpRegion);
     free((char *)regRects);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->PolyFillRect) (pDrawable, pGC, nrects, rects);
 
-  if (nrects)
+  if (nrects) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1330,12 +1452,19 @@ static void rfbPolyFillArc(DrawablePtr pDrawable, GCPtr pGC, int narcs,
 
     REGION_DESTROY(pDrawable->pScreen, tmpRegion);
     free((char *)rects);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->PolyFillArc) (pDrawable, pGC, narcs, arcs);
 
-  if (narcs)
+  if (narcs) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1405,12 +1534,19 @@ static int rfbPolyText8(DrawablePtr pDrawable, GCPtr pGC, int x, int y,
     TRC_REGION(pDrawable->pScreen, &tmpRegion, "rfbPolyText8() mod");
 
     REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   ret = (*pGC->ops->PolyText8) (pDrawable, pGC, x, y, count, chars);
 
-  if (count)
+  if (count) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
   return ret;
@@ -1446,12 +1582,19 @@ static int rfbPolyText16(DrawablePtr pDrawable, GCPtr pGC, int x, int y,
     TRC_REGION(pDrawable->pScreen, &tmpRegion, "rfbPolyText16() mod");
 
     REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   ret = (*pGC->ops->PolyText16) (pDrawable, pGC, x, y, count, chars);
 
-  if (count)
+  if (count) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
   return ret;
@@ -1486,12 +1629,19 @@ static void rfbImageText8(DrawablePtr pDrawable, GCPtr pGC, int x, int y,
     TRC_REGION(pDrawable->pScreen, &tmpRegion, "rfbImageText8() mod");
 
     REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->ImageText8) (pDrawable, pGC, x, y, count, chars);
 
-  if (count)
+  if (count) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1525,12 +1675,19 @@ static void rfbImageText16(DrawablePtr pDrawable, GCPtr pGC, int x, int y,
     TRC_REGION(pDrawable->pScreen, &tmpRegion, "rfbImageText16() mod");
 
     REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->ImageText16) (pDrawable, pGC, x, y, count, chars);
 
-  if (count)
+  if (count) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1566,12 +1723,19 @@ static void rfbImageGlyphBlt(DrawablePtr pDrawable, GCPtr pGC, int x, int y,
     TRC_REGION(pDrawable->pScreen, &tmpRegion, "rfbImageGlyphBlt() mod");
 
     REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->ImageGlyphBlt) (pDrawable, pGC, x, y, nglyph, ppci, pglyphBase);
 
-  if (nglyph)
+  if (nglyph) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1607,12 +1771,19 @@ static void rfbPolyGlyphBlt(DrawablePtr pDrawable, GCPtr pGC, int x, int y,
     TRC_REGION(pDrawable->pScreen, &tmpRegion, "rfbPolyGlyphBlt() mod");
 
     REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
+#ifdef DRI3
+    rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
   }
 
   (*pGC->ops->PolyGlyphBlt) (pDrawable, pGC, x, y, nglyph, ppci, pglyphBase);
 
-  if (nglyph)
+  if (nglyph) {
+#ifdef DRI3
+    rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
     SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
+  }
 
   GC_OP_EPILOGUE(pGC);
 }
@@ -1650,8 +1821,16 @@ static void rfbPushPixels(GCPtr pGC, PixmapPtr pBitMap, DrawablePtr pDrawable,
 
   REGION_UNINIT(pDrawable->pScreen, &tmpRegion);
 
+#ifdef DRI3
+  rfbDRI3SyncBOToDrawable(pDrawable);
+#endif
+
   (*pGC->ops->PushPixels) (pGC, pBitMap, pDrawable, w, h, x, y);
 
+#ifdef DRI3
+  rfbDRI3SyncDrawableToBO(pDrawable);
+#endif
+
   SCHEDULE_FB_UPDATE(pDrawable->pScreen, prfb);
 
   GC_OP_EPILOGUE(pGC);
@@ -1696,11 +1875,19 @@ void rfbComposite(CARD8 op, PicturePtr pSrc, PicturePtr pMask, PicturePtr pDst,
     TRC_REGION(pScreen, &tmpRegion, "rfbComposite() mod");
   }
 
+#ifdef DRI3
+  rfbDRI3SyncBOToDrawable(pDst->pDrawable);
+#endif
+
   ps->Composite = prfb->Composite;
   (*ps->Composite) (op, pSrc, pMask, pDst, xSrc, ySrc, xMask, yMask,
                     xDst, yDst, width, height);
   ps->Composite = rfbComposite;
 
+#ifdef DRI3
+  rfbDRI3SyncDrawableToBO(pDst->pDrawable);
+#endif
+
   if (is_visible(pDst->pDrawable)) {
     SCHEDULE_FB_UPDATE(pScreen, prfb);
 
@@ -1797,15 +1984,141 @@ void rfbGlyphs(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
     REGION_DESTROY(pScreen, tmpRegion);
   }
 
+#ifdef DRI3
+  rfbDRI3SyncBOToDrawable(pDst->pDrawable);
+#endif
+
   ps->Glyphs = prfb->Glyphs;
   (*ps->Glyphs) (op, pSrc, pDst, maskFormat, xSrc, ySrc, nlists, lists,
                  glyphs);
   ps->Glyphs = rfbGlyphs;
 
+#ifdef DRI3
+  rfbDRI3SyncDrawableToBO(pDst->pDrawable);
+#endif
+
   if (is_visible(pDst->pDrawable))
     SCHEDULE_FB_UPDATE(pScreen, prfb);
 }
 
+
+#ifdef DRI3
+
+void rfbCompositeRects(CARD8 op, PicturePtr pDst, xRenderColor *color,
+                       int nRect, xRectangle *rects)
+{
+  ScreenPtr pScreen = pDst->pDrawable->pScreen;
+  rfbFBInfoPtr prfb = &rfbFB;
+  PictureScreenPtr ps = GetPictureScreen(pScreen);
+
+  TRC("rfbCompositeRects() op=%d Dst=0x%.8x nRect=%d\n", op,
+      XID(pDst->pDrawable), nRect);
+
+  if (nRect)
+    rfbDRI3SyncBOToDrawable(pDst->pDrawable);
+
+  ps->CompositeRects = prfb->CompositeRects;
+  (*ps->CompositeRects) (op, pDst, color, nRect, rects);
+  ps->CompositeRects = rfbCompositeRects;
+
+  if (nRect)
+    rfbDRI3SyncDrawableToBO(pDst->pDrawable);
+}
+
+
+void rfbTrapezoids(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
+                   PictFormatPtr maskFormat, INT16 xSrc, INT16 ySrc, int ntrap,
+                   xTrapezoid *traps)
+{
+  ScreenPtr pScreen = pDst->pDrawable->pScreen;
+  rfbFBInfoPtr prfb = &rfbFB;
+  PictureScreenPtr ps = GetPictureScreen(pScreen);
+
+  TRC("rfbTrapezoids() op=%d Src=0x%.8x Dst=0x%.8x xSrc=%d ySrc=%d ntrap=%d\n",
+      op, XID(pSrc->pDrawable), XID(pDst->pDrawable), xSrc, ySrc, ntrap);
+
+  if (ntrap)
+    rfbDRI3SyncBOToDrawable(pDst->pDrawable);
+
+  ps->Trapezoids = prfb->Trapezoids;
+  (*ps->Trapezoids) (op, pSrc, pDst, maskFormat, xSrc, ySrc, ntrap, traps);
+  ps->Trapezoids = rfbTrapezoids;
+
+  if (ntrap)
+    rfbDRI3SyncDrawableToBO(pDst->pDrawable);
+}
+
+
+void rfbTriangles(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
+                  PictFormatPtr maskFormat, INT16 xSrc, INT16 ySrc, int ntri,
+                  xTriangle *tris)
+{
+  ScreenPtr pScreen = pDst->pDrawable->pScreen;
+  rfbFBInfoPtr prfb = &rfbFB;
+  PictureScreenPtr ps = GetPictureScreen(pScreen);
+
+  TRC("rfbTriangles() op=%d Src=0x%.8x Dst=0x%.8x xSrc=%d ySrc=%d ntri=%d\n",
+      op, XID(pSrc->pDrawable), XID(pDst->pDrawable), xSrc, ySrc, ntri);
+
+  if (ntri)
+    rfbDRI3SyncBOToDrawable(pDst->pDrawable);
+
+  ps->Triangles = prfb->Triangles;
+  (*ps->Triangles) (op, pSrc, pDst, maskFormat, xSrc, ySrc, ntri, tris);
+  ps->Triangles = rfbTriangles;
+
+  if (ntri)
+    rfbDRI3SyncDrawableToBO(pDst->pDrawable);
+}
+
+
+void rfbTriStrip(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
+                 PictFormatPtr maskFormat, INT16 xSrc, INT16 ySrc, int npoint,
+                 xPointFixed *points)
+{
+  ScreenPtr pScreen = pDst->pDrawable->pScreen;
+  rfbFBInfoPtr prfb = &rfbFB;
+  PictureScreenPtr ps = GetPictureScreen(pScreen);
+
+  TRC("rfbTriStrip() op=%d Src=0x%.8x Dst=0x%.8x xSrc=%d ySrc=%d npoint=%d\n",
+      op, XID(pSrc->pDrawable), XID(pDst->pDrawable), xSrc, ySrc, npoint);
+
+  if (npoint)
+    rfbDRI3SyncBOToDrawable(pDst->pDrawable);
+
+  ps->TriStrip = prfb->TriStrip;
+  (*ps->TriStrip) (op, pSrc, pDst, maskFormat, xSrc, ySrc, npoint, points);
+  ps->TriStrip = rfbTriStrip;
+
+  if (npoint)
+    rfbDRI3SyncDrawableToBO(pDst->pDrawable);
+}
+
+
+void rfbTriFan(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
+               PictFormatPtr maskFormat, INT16 xSrc, INT16 ySrc, int npoint,
+               xPointFixed *points)
+{
+  ScreenPtr pScreen = pDst->pDrawable->pScreen;
+  rfbFBInfoPtr prfb = &rfbFB;
+  PictureScreenPtr ps = GetPictureScreen(pScreen);
+
+  TRC("rfbTriFan() op=%d Src=0x%.8x Dst=0x%.8x xSrc=%d ySrc=%d npoint=%d\n",
+      op, XID(pSrc->pDrawable), XID(pDst->pDrawable), xSrc, ySrc, npoint);
+
+  if (npoint)
+    rfbDRI3SyncBOToDrawable(pDst->pDrawable);
+
+  ps->TriFan = prfb->TriFan;
+  (*ps->TriFan) (op, pSrc, pDst, maskFormat, xSrc, ySrc, npoint, points);
+  ps->TriFan = rfbTriFan;
+
+  if (npoint)
+    rfbDRI3SyncDrawableToBO(pDst->pDrawable);
+}
+
+#endif  /* DRI3 */
+
 #endif  /* RENDER */
 
 
