@@ -2,7 +2,7 @@
  * rfb.h - header file for RFB DDX implementation.
  */
 
-/* Copyright (C) 2010-2022, 2024 D. R. Commander.  All Rights Reserved.
+/* Copyright (C) 2010-2025 D. R. Commander.  All Rights Reserved.
  * Copyright (C) 2011, 2015 Pierre Ossman for Cendio AB.  All Rights Reserved.
  * Copyright (C) 2011 Joel Martin
  * Copyright (C) 2011 Gernot Tenchio
@@ -87,7 +87,7 @@
 
 /* Maximum number of threads to use for multithreaded encoding, regardless of
    the CPU count */
-#define MAX_ENCODING_THREADS 8
+#define MAX_ENCODING_THREADS 4
 
 /* Maximum number of client connections.  The default of 100 should be more
    than enough for most use cases.  The ceiling is set to 500 to give us plenty
@@ -173,12 +173,22 @@ typedef struct {
   /* wrapped screen functions */
 
   CloseScreenProcPtr                    CloseScreen;
+#ifdef DRI3
+  SourceValidateProcPtr                 SourceValidate;
+#endif
   CreateGCProcPtr                       CreateGC;
   CopyWindowProcPtr                     CopyWindow;
   ClearToBackgroundProcPtr              ClearToBackground;
 #ifdef RENDER
   CompositeProcPtr                      Composite;
   GlyphsProcPtr                         Glyphs;
+#ifdef DRI3
+  CompositeRectsProcPtr                 CompositeRects;
+  TrapezoidsProcPtr                     Trapezoids;
+  TrianglesProcPtr                      Triangles;
+  TriStripProcPtr                       TriStrip;
+  TriFanProcPtr                         TriFan;
+#endif
 #endif
   InstallColormapProcPtr                InstallColormap;
   UninstallColormapProcPtr              UninstallColormap;
@@ -432,6 +442,10 @@ typedef struct rfbClientRec {
                                        extension */
   Bool enableVMwareLEDState;        /* client supports VMware LED State
                                        extension */
+  Bool enableTightWithoutZlib;      /* client supports Tight Encoding Without
+                                       Zlib extension */
+  Bool enableExtMouseButtons;       /* client supports Extended Mouse Buttons
+                                       extension */
   Bool useRichCursorEncoding;       /* rfbEncodingRichCursor is preferred */
   Bool cursorWasChanged;            /* cursor shape update should be sent */
   Bool cursorWasMoved;              /* cursor position update should be sent */
@@ -481,6 +495,8 @@ typedef struct rfbClientRec {
   Bool pendingDesktopResize, pendingExtDesktopResize;
   int reason, result;
 
+  Bool pendingExtMouseButtonsRect;
+
   /* Server-side key mapping */
   Bool pendingQEMUExtKeyEventRect, pendingLEDState;
   unsigned int ledState;
@@ -821,13 +837,32 @@ extern void rfbComposite(CARD8 op, PicturePtr pSrc, PicturePtr pMask,
                          PicturePtr pDst, INT16 xSrc, INT16 ySrc, INT16 xMask,
                          INT16 yMask, INT16 xDst, INT16 yDst, CARD16 width,
                          CARD16 height);
-
 extern void rfbGlyphs(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
                       PictFormatPtr maskFormat, INT16 xSrc, INT16 ySrc,
                       int nlists, GlyphListPtr lists, GlyphPtr *glyphs);
+#ifdef DRI3
+extern void rfbCompositeRects(CARD8 op, PicturePtr pDst, xRenderColor *color,
+                              int nRect, xRectangle *rects);
+extern void rfbTrapezoids(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
+                          PictFormatPtr maskFormat, INT16 xSrc, INT16 ySrc,
+                          int ntrap, xTrapezoid *traps);
+extern void rfbTriangles(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
+                         PictFormatPtr maskFormat, INT16 xSrc, INT16 ySrc,
+                         int ntri, xTriangle *tris);
+extern void rfbTriStrip(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
+                        PictFormatPtr maskFormat, INT16 xSrc, INT16 ySrc,
+                        int npoint, xPointFixed *points);
+extern void rfbTriFan(CARD8 op, PicturePtr pSrc, PicturePtr pDst,
+                      PictFormatPtr maskFormat, INT16 xSrc, INT16 ySrc,
+                      int npoint, xPointFixed *points);
+#endif
 #endif
 
 extern Bool rfbCloseScreen(ScreenPtr);
+#ifdef DRI3
+extern void rfbSourceValidate(DrawablePtr pDrawable, int x, int y, int width,
+                              int height, unsigned int subWindowMode);
+#endif
 extern Bool rfbCreateGC(GCPtr);
 extern void rfbPaintWindowBackground(WindowPtr, RegionPtr, int what);
 extern void rfbPaintWindowBorder(WindowPtr, RegionPtr, int what);
@@ -837,6 +872,17 @@ extern void rfbClearToBackground(WindowPtr, int x, int y, int w, int h,
 extern RegionPtr rfbRestoreAreas(WindowPtr, RegionPtr);
 
 
+/* dri3.c */
+
+#ifdef DRI3
+extern const char *driNode;
+
+extern Bool rfbDRI3Initialize(ScreenPtr pScreen);
+extern void rfbDRI3SyncBOToDrawable(DrawablePtr drawable);
+extern void rfbDRI3SyncDrawableToBO(DrawablePtr drawable);
+#endif
+
+
 /* flowcontrol.c */
 
 extern void rfbInitFlowControl(rfbClientPtr cl);
@@ -905,8 +951,6 @@ extern void KeyEvent(KeySym keySym, Bool down);
 extern void KbdReleaseAllKeys(void);
 extern void QEMUExtKeyboardEventInit(void);
 
-extern Bool stristr(const char *s1, const char *s2);
-
 
 /* nvctrlext.c */
 
